@@ -36,7 +36,8 @@ class Carrier(ModelSQL, ModelView):
         'Compute carrier sale price with currency'
         User = Pool().get('res.user')
         if self.carrier_cost_method == 'product':
-            user = User(Transaction().user)
+            user = User(Transaction().user
+                or Transaction().context.get('user'))
             return self.carrier_product.list_price, user.company.currency.id
         return 0, None
 
@@ -114,8 +115,7 @@ class CarrierSelection(
                 carriers = Carrier.search([])
         else:
             for selection in selections:
-                if (selection.match(pattern)
-                        and selection.carrier not in carriers):
+                if selection.match(pattern):
                     carriers.append(selection.carrier)
 
         cls._get_carriers_cache.set(key, list(map(int, carriers)))
